@@ -1,12 +1,13 @@
-{-# LANGUAGE OverlappingInstances, UndecidableInstances #-}
+{-# LANGUAGE FlexibleContexts, FlexibleInstances, MultiParamTypeClasses, OverlappingInstances, TypeSynonymInstances, UndecidableInstances, TypeFamilies #-}
 module HJScript.XMLGenerator (
 --        ToChildNodes(..), ToAttributeNode(..),
-        
-        genElement, genEElement, asChild, asAttr, Attr(..)
+
+        XML, Child, Attribute, genElement, genEElement, asChild, asAttr, fromStringLit, Attr(..)
         ) where
 
 --import qualified HSX.XMLGenerator as HSX (XMLGen(..))
-import HSX.XMLGenerator
+import qualified Data.Text.Lazy as TL
+import HSP.XMLGenerator
 
 import HJScript.Monad
 import HJScript.Lang
@@ -23,6 +24,7 @@ type Attribute = Exp AttributeNode
 
 instance XMLGen HJScript' where
  type XMLType          HJScript' = XML
+ type StringType       HJScript' = String
  newtype ChildType     HJScript' = HJSChild Child
  newtype AttributeType HJScript' = HJSAttr Attribute
  genElement = element
@@ -30,9 +32,12 @@ instance XMLGen HJScript' where
  xmlToChild = HJSChild . castToNode
  pcdataToChild str = HJSChild . castToNode $ document # createTextNode (string str)
 
-element :: (EmbedAsChild HJScript' c, 
-            EmbedAsAttr HJScript' a) 
-            => Name -> [a] -> [c] -> HJScript XML
+fromStringLit :: String -> String
+fromStringLit = id
+
+element :: (EmbedAsChild HJScript' c,
+            EmbedAsAttr HJScript' a)
+            => Name String -> [a] -> [c] -> HJScript XML
 element (ns, ln) atts xmls = do
   let name = (maybe id (\x y -> y ++ ':':x) ns) ln
   elem <- fmap val $ varWith $ document # createElement (string name)
@@ -42,9 +47,14 @@ element (ns, ln) atts xmls = do
   mapM (\child -> elem # appendChild child) $ map stripChild cxml
   return elem
 
-eElement :: EmbedAsAttr HJScript' a => Name -> [a] -> HJScript XML
+eElement :: EmbedAsAttr HJScript' a => Name String -> [a] -> HJScript XML
 eElement n attrs = element n attrs ([] :: [Child])
-
+{-
+instance EmbedAsAttr HJScript' (Attr TL.Text TL.Text) where
+    asAttr (n := v) = asAttr (TL.unpack n := TL.unpack v)
+-}
+instance EmbedAsChild HJScript' TL.Text where
+    asChild t = asChild (TL.unpack t)
 
 instance XMLGenerator HJScript'
 
@@ -71,10 +81,10 @@ instance EmbedAsChild HJScript' XML where
 instance EmbedAsAttr HJScript' Attribute where
  asAttr = asAttr . HJSAttr
 
-instance (IsName n, IsAttrNodeValue a) => EmbedAsAttr HJScript' (Attr n a) where
+instance (IsAttrNodeValue a, IsName n TL.Text) => EmbedAsAttr HJScript' (Attr n a) where
  asAttr (k := a) = asAttr $ do
     let (ns, ln) = toName k
-        name = (maybe id (\x y -> y ++ ':':x) ns) ln
+        name = (maybe id (\x y -> y ++ ':':x) (fmap TL.unpack ns)) (TL.unpack ln)
     v <- toAttrNodeValue a
     an <- inVar $ document # createAttribute (string name)
     an # value .=. v
@@ -85,7 +95,11 @@ class IsAttrNodeValue a where
  toAttrNodeValue :: a -> HJScript JString
 
 instance JShow a => IsAttrNodeValue a where
- toAttrNodeValue = return . jshow
+    toAttrNodeValue = return . jshow
+
+instance IsAttrNodeValue TL.Text where
+    toAttrNodeValue = return . jshow . TL.unpack
+
 
 instance IsAttrNodeValue a => IsAttrNodeValue (HJScript a) where
  toAttrNodeValue = (>>= toAttrNodeValue)
